' *********************************************************
' PLX_DAQ_Controller, PLX_DAQ_Table, PLX_DAQ_Matrix Classes
' 
' 
' The classes published here provide a high-level ZBASIC interface for 
' data acquisition in MS-Excel using PLX_DAQ (Parallax Inc)
'
' V1.3 (Overloading ambiguity problem resolved that appeared with V3.3.0)
'
' Jurjen Kranenborg
' Uppsala, November 2010
'
' 

Option Objects
Option Base 1


' =========================================================
' Class PDX_DAQ_Controller implements a number of methods 
' for reading and/or manipulating the four checkbox settings
' on the controller window as well as displaying user-defined messages
' Together with PLX_DAQ_Table or PLX_DAQ_Matrix an MS-Excel 
' worksheet-based interactive user interface can be defined.
'
Class PLX_DAQ_Controller

Public:

	Sub _Create()
	
		Register.Console.Echo = False
	
	End Sub
	
	
	Sub _Destroy()
	
		Register.Console.Echo = True
		
	End Sub


	' -----------------------------------------------------
	' Test whether one of the checkboxes (1 - 4) of the PLX-DAQ 
	' controller has the status according to the "Checked"
	' paramaters. If so, this method gives True, otherwise False
	'
	Function ReadCheckBoxStatus(ByVal TickBoxNumber as Byte, ByVal Checked as Boolean) as Boolean
	
		Dim ReceivedText as String
		
		Select Case TickBoxNumber
		Case 1
			Console.WriteLine("DOWNLOAD,GET")
		Case 2
			Console.WriteLine("STORED,GET")
		Case 3
			Console.WriteLine("USER1,GET")
		Case 4 
			Console.WriteLine("USER2,GET")
		End Select
		
		Delay(0.1)
		ReceivedText = Chr(Console.Read)
		
		ReadCheckBoxStatus = False
		If ReceivedText = "1" And Checked = True Then 
			ReadCheckBoxStatus = True
		End If
		If ReceivedText = "0" And Checked = False Then 
			ReadCheckBoxStatus = True
		End If
		
		Call ClearQueue(CByteArray(Register.RxQueue))
	
	End Function
	
	
	' -----------------------------------------------------
	' Present the actual status of all checkboxes (1-4) in
	' parameter "BoxBits" (bits 0 - 3)
	'
	Function ReadCheckBoxes(ByRef BoxBits as Byte) as Boolean
	
		BoxBits = &B00000000
		ReadCheckBoxes = False
		
		If ReadCheckBoxStatus(1,True) Then 
			BoxBits = BoxBits Or &B00000001
		End If
		If ReadCheckBoxStatus(2,True) Then 
			BoxBits = BoxBits Or &B00000010
		End If
		If ReadCheckBoxStatus(3,True) Then 
			BoxBits = BoxBits Or &B00000100
		End If
		If ReadCheckBoxStatus(4,True) Then 
			BoxBits = BoxBits Or &B00001000
		End If
		If BoxBits <> &B00000000 Then
			ReadCheckBoxes = True
		End If
		
	End Function
	
	
	' -----------------------------------------------------
	' Set the indicated checkbox (1-4) according to the status
	' in the "Checked" parameters
	'
	Sub SetCheckBoxStatus(ByVal CheckBoxNumber as Byte, ByVal Checked as Boolean)
	
		Dim StateValue as Byte = 0
		
		If Checked Then
			StateValue = 1
		End If
		
		Select Case CheckBoxNumber
		Case 1
			Console.WriteLine("DOWNLOAD,SET," & CStr(StateValue))
		Case 2
			Console.WriteLine("STORED,SET," & CStr(StateValue))
		Case 3
			Console.WriteLine("USER1,SET," & CStr(StateValue))
		Case 4 
			Console.WriteLine("USER2,SET," & CStr(StateValue))
		End Select
		
	End Sub
	
	
	' -----------------------------------------------------
	' Uncheck all checkboxes
	'
	Sub ResetCheckBoxes()
	
		Console.WriteLine("DOWNLOAD,SET,0")
		Console.WriteLine("STORED,SET,0")
		Console.WriteLine("USER1,SET,0")
		Console.WriteLine("USER2,SET,0")
	
	End Sub
	
	
	' -----------------------------------------------------
	' Define default label and message text of the PLX-DAQ
	' controller. These texts are stored in the object and 
	' shown when method UpdateController is invoked
	'
	Sub SetControllerProperties(ByVal name1 as String = "", ByVal name2 as String = "", _ 
						  ByVal name3 as String = "", ByVal name4 as String = "", _
						  ByVal message as String = "")
	
		Label_CheckBox1 = name1
		Label_CheckBox2 = name2
		Label_CheckBox3 = name3
		Label_CheckBox4 = name4
		DefaultControllerMessage = message
	
	End Sub
	
	
	' -----------------------------------------------------
	' Show default controller messages
	'
	Sub UpdateController()
	
		Console.WriteLine("DOWNLOAD,LABEL," & Label_CheckBox1)
		Console.WriteLine("STORED,LABEL," & Label_CheckBox2)
		Console.WriteLine("USER1,LABEL," & Label_CheckBox3)
		Console.WriteLine("USER2,LABEL," & Label_CheckBox4)
		Console.WriteLine("MSG," & DefaultControllerMessage)
	
	End Sub
	
	
	' -----------------------------------------------------
	' Show supplied controller message
	'
	Sub WriteControllerMessage(ByVal message as String)
	
		Console.WriteLine("MSG," & message)
		
	End Sub
	
	
Protected:

	Dim Label_CheckBox1 as String
	Dim Label_CheckBox2 as String
	Dim Label_CheckBox3 as String
	Dim Label_CheckBox4 as String
	Dim DefaultControllerMessage as String
	
End Class


' =========================================================
' Class PLX_DAQ_Table implements a large number of methods
' for creating, managing and reading of tables in an MS-Excel worksheet 
' and keeps state variables for easy table management in user programs.
' The focus is on numerical applications like datalogging etc. where
' large number of floating-point data (type: Single) need to be 
' managed.
'
' An object of this class describes the name of an MS-ExCel table, 
' its parameter names and the size of the table (number of parameters, 
' number of rows/records). An MS-Excel table at position 
' row = i and column J is assumed always to be formatted as follows:
' 
' 	.	j		j+1		j+2		.		...
'
' 	i	TableName		.		.		...
'       -----------------------------------
'
' i+1	Label1	Label2	Label3	Label4	...
' 	    -----------------------------------
'
' i+2	Value11	Value12	Value13	Value14	...
' 
' 	.	Value21	Value22	Value23 Value24	...
'
'   ..	..		..		..		..		...
'
'
' Additionally the class provides the following methods:
'
' * Preparing a MS-Excel worksheet
'	- ClearSheet
' * Manipulating individual worksheet cells (strings):
'	- WriteCell
'	- ReadCell
' * defining a table (properties only, not actual content):
'	- SetTableProperties 
'	- ChangeTablePosition
' * writing/reading table content
'	- WriteTable (from a 2D Single array)
'	- WriteTableRecord (row, from a 1D Single array)
'	- Rewind (for use with WriteTableRecord)
'	- ReadTable (into a 2D Single array)
' 	- ReadTableRecord (row, into a 1D Single array)
'	- Duplicate (copy a table to another place)
'	- ClearTableData (clear table numerical contents)
'
' Several MS-Excel tables may be maintained at the same time 
' by creating several objects of this class. Using ChangeTablePosition
' these tables may be accessed interchangeably.
'
' Due to the internal storage scheme of ZBASIC arrays the 2D
' arrays are organized as array(1.. NumPars, 1 ... NumRecords)
' to allow easy access, i.e; MS-Excel Value14 = array(4,1)
'
' A special table variant called matrix is implemented by 
' subclass PLX_DAQ_Matrix which includes labels at the 
' left side of the table (see the  subclass description)
'
Class PLX_DAQ_Table 

Public:

	Sub _Create()
	
		Register.Console.Echo = False
	
	End Sub


	Sub _Destroy()
	
		Register.Console.Echo = True
		
	End Sub


	' -----------------------------------------------------
	' Clear all data in MS-Excel worksheet 
	' (except for first row)
	'
	Sub ClearSheet()

		Console.WriteLine("CLEARDATA")
		
	End Sub
	
	
	' -----------------------------------------------------
	' Basic cell write operation: write text string to 
	' indicated cell. This method forms the basis of most of 
	' the other method implementations
	'
	Sub WriteCell(ByVal RowIndex as UnsignedInteger, Byval ColumnIndex as UnsignedInteger, ByVal CellText as String)
	
		Console.WriteLine("CELL,SET," & ExcelTableIndex(RowIndex, ColumnIndex) & "," & CellText)
	
	End Sub
	
	
	' -----------------------------------------------------
	' Basic cell read operation: read indicated cell contents
	' into string. This method forms the basis of most of 
	' the other method implementations
	'
	Function ReadCell(ByVal RowIndex as UnsignedInteger, Byval ColumnIndex as UnsignedInteger) as String
		
		Dim CellId as String
		Dim ReceivedText as String = ""
		Dim ReceivedChar as String = ""
				
		CellId = ExcelTableIndex(RowIndex, ColumnIndex)

		Console.WriteLine("CELL,GET," & CellId)

		Do
	
			ReceivedChar = Chr(Console.Read)
			If Asc(ReceivedChar) <> 13 Then
				ReceivedText = ReceivedText & ReceivedChar
			End If
		
		Loop Until Asc(ReceivedChar) = 13

		Call ClearQueue(CByteArray(Register.RxQueue))

		ReadCell = ReceivedText
	
	End Function
	
	
	' -----------------------------------------------------
	' Define the parameters that describe the MS-Excel table
	' (The numerical values in the table are NOT stored in the object)
	' These parameters are needed by all table-related methods
	' See the table layout description at the beginning of the 
	' class definition
	'
	Sub SetTableProperties(ByVal TableNameString as String,  _
					  ByVal NumberOfPars as UnsignedInteger, ByVal NumberOfRecords as UnsignedInteger, _
					  ByVal TableRowPosition as UnsignedInteger, ByVal TableColumnPosition as UnsignedInteger, _
					  ByVal TableParameterLabels as String = "")
					  
		TableStart_Row = TableRowPosition			' Row positon of the table
		TableStart_Column = TableColumnPosition		' Column position of the table
		TableName = TableNameString
		ParLabels = TableParameterLabels
		NumPars = NumberOfPars
		NumRecords = NumberOfRecords
	
	End Sub
	
	
	' -----------------------------------------------------
	' Change the position definiton of the table
	' This allows several MS-Excel tables to easily selected 
	' by choice of start position
	'
	Sub ChangeTablePosition(Byval TableRowPos as UnsignedInteger, ByVal TableColumnPos as UnsignedInteger)
	
		TableStart_Row = TableRowPos
		TableStart_Column = TableColumnPos 
		
	End Sub
	
	
	' -----------------------------------------------------
	' Write a 2D array (dimensions: (1..NumPars, 1..NumRecords),
	' type: Single) to the corresponding MS-Excel table
	'
	Sub WriteTable(ByVal TablePointer as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars*MaxNumRecords) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		
		Call WriteCell(TableStart_Row, TableStart_Column, TableName)
		Call GenerateParameterLabels
		
		For CurrentRecordIndex = 1 To NumRecords
			For ParNumber = 1 To NumPars
				Call WriteCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber - 1, CStr(DataRecordArray((CurrentRecordIndex-1)*NumPars + ParNumber)))
			Next ParNumber
		Next CurrentRecordIndex
		
		NumRecordsWritten = NumRecords
	
	End Sub
	
	
	' -----------------------------------------------------
	' Write a Single 1D array (dimensions: (2..NumPars) to the 
	' record with indicated row number in the MS-Excel table.
	' The first parameter written is CurrentXvalue, then the array
	' elements follow.
	'
	Sub WriteTableRecord(ByVal RecordPointer as UnsignedInteger, _ 
						ByVal CurrentXvalue as Single = 0.0)
	
		Dim DataRecordArray(2 To MaxPars) as Single Based RecordPointer
		Dim ParNumber as unsignedInteger
		
		RecordedTable = True
		
		If NumRecordsWritten = 0 Then
			Call WriteCell(TableStart_Row, TableStart_Column, TableName)
			Call GenerateParameterLabels
		End If

		Call WriteCell(TableStart_Row + 2 + NumRecordsWritten, TableStart_Column, CStr(CurrentXvalue))
		For ParNumber = 2 To NumPars
			Call WriteCell(TableStart_Row + 2 + NumRecordsWritten, TableStart_Column + ParNumber - 1, CStr(DataRecordArray(ParNumber)))
		Next ParNumber
	
		NumRecordsWritten = NumRecordsWritten + 1
	
	End Sub
	
	
	' -----------------------------------------------------
	' Reset the internal row pointer to allow individual row
	' records to be written (only to be used in conjuction
	' with WriteTableRecord method)
	'
	Sub Rewind()
	
		NumRecordsWritten = 0
	
	End Sub


	' -----------------------------------------------------
	' Read a 2D array (dimensions: (1..NumPars, 1..NumRecords),
	' type: Single) from the corresponding MS-Excel table
	'
	Sub ReadTable(ByVal TablePointer as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars*MaxNumRecords) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		Dim Flag as Boolean
		
		For CurrentRecordIndex = 1 To NumRecordsWritten
			For ParNumber = 1 To NumPars
				Call ValueS(ReadCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber - 1), DataRecordArray((CurrentRecordIndex-1)*NumPars + ParNumber), flag)
			Next ParNumber
		Next CurrentRecordIndex
	
	End Sub
	
	
	' -----------------------------------------------------
	' Read a Single 1D array (dimensions: (1..NumPars) from the 
	' record with indicated row number in the MS-Excel table
	'
	Sub ReadTableRecord(ByVal TablePointer as UnsignedInteger, ByVal RowIndex as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim Flag as Boolean
		
		For ParNumber = 1 To NumPars
			Call ValueS(ReadCell(TableStart_Row + 2 + RowIndex - 1, TableStart_Column + ParNumber - 1), DataRecordArray(ParNumber), Flag)
		Next ParNumber
	
	End Sub
	
	
	' -----------------------------------------------------
	' Duplicate an MS-Excel table to another MS-Excel table
	' without any use of intermediate ZBASIC array storage 
	' Optionally a new table name can be supplied.
	'
	Sub Duplicate(ByVal TableRow as UnsignedInteger, ByVal TableColumn as UnsignedInteger, ByVal AltName as String = "")
	
		Dim i as UnsignedInteger
		Dim j as UnsignedInteger
		
		If AltName = "" Then
			Call WriteCell(TableRow, TableColumn, ReadCell(TableStart_Row, TableStart_Column))
		Else
			Call WriteCell(TableRow, TableColumn, AltName)
		End If
	
		For j = 0 To NumRecordsWritten
			For i = 1 To Numpars
				Call WriteCell(TableRow + j + 1, TableColumn + i - 1, ReadCell(TableStart_Row + j + 1, TableStart_Column + i - 1))
			Next i
		Next j
	
	End sub
	
	
	' -----------------------------------------------------
	' Clear the numerical contents of an MS_Excel table
	'
	Sub ClearTableData(ByVal EmptyString as String = "-")
	
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		
		For CurrentRecordIndex = 1 To NumRecords
			For ParNumber = 1 To NumPars
				Call WriteCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber - 1, EmptyString)
			Next ParNumber
		Next CurrentRecordIndex
		
	End Sub
	
	
Protected:

	' -----------------------------------------------------
	' Generate labels on the second row of the table:
	' - if ParLabels is empty then "Par i" type labels 
	'   are generated automatically in each column i,
	' - if ParLabels is a comma-separated string then
	'   the labels between commas are put in each table column
	' Labels are defined using SetTableProperties
	'
	Sub GenerateParameterLabels()
		
		Dim ColonIndex as Byte
		Dim TextList as String = ParLabels
		Dim PickLabel as String
		Dim ColumnPointer as UnsignedInteger = TableStart_Column
		Dim i as UnsignedInteger
	
		If ParLabels = "" then
			For i = 1 To NumPars 
				Call WriteCell(TableStart_Row + 1, TableStart_Column + i - 1, "Par " & CStr(i - 1))
			Next i
		Else
			Do
				ColonIndex = Strfind(TextList,",")
				If ColonIndex <> 0 Then
					PickLabel = Left(TextList, ColonIndex - 1)
					Call WriteCell(TableStart_Row + 1, ColumnPointer, PickLabel)
					ColumnPointer = ColumnPointer + 1
					TextList = Mid(TextList, ColonIndex + 1)
				Else
					Call WriteCell(TableStart_Row + 1, ColumnPointer, TextList)
				End If
			Loop Until ColonIndex = 0
		End If
		
	End Sub
	
	
	' -----------------------------------------------------
	' Generate MS-Excel table index
	' Example: ExcelTableIndex(5,2) (row 5, column 2) gives 
	' the string B5 for the corresponding MS-Excel address
	'
	Function ExcelTableIndex(ByVal TableRow as UnsignedInteger, ByVal TableColumn as UnsignedInteger) as String
	
		ExcelTableIndex = Chr(TableColumn + CUInt(Asc("A")) - 1) & CStr(TableRow) 
		
	End Function
	
	
	' -----------------------------------------------------
	' Object state variables describing the properties of
	' the corresponding MS-Excel table
	' These state variables are used in all methods
	'
	Dim TableStart_Row as UnsignedInteger
	Dim TableStart_Column as UnsignedInteger

	Dim TableName as String
	Dim ParLabels as String
	
	Dim NumPars as UnsignedInteger
	Dim NumRecords as UnsignedInteger

	Dim RecordedTable as Boolean = False
	Dim NumRecordsWritten as UnsignedInteger = 0

	Const MaxPars as UnsignedInteger = 26
	Const MaxNumRecords as UnsignedInteger = 65535


End Class


' =========================================================
' Class PLX_DAQ_Matrix is a subclass of PLX_DAQ_Table
' defining a matrix as a special type of table that has
' a left column with item labels as well. Because of this,
' the numerical values in the table are shifted to the right
' by one column. Furthermore, WriteTableRecord shall supply 
' the item label. The affected table methods in PLX_DAQ_table 
' are overloaded to take care of these changes automatically, all other 
' methods of PLX_DAQ_Table (like the basic ReadCell and WriteCell methods)
' are available as well.
'
' A matrix is formatted in an MS-Excel worksheet as follows:
'
' .		j		j+1		j+2		.		.		.
'
' i		TableName		.		.		.		.
'       ...........................................
'
' i+1	Label0	Label1	Label2	Label3	Label4	.
'       ...........................................
' 			  .
' i+2	Item1 .	Value11	Value12	Value13	Value14	.
'             .
' .		Item2 .	Value21	Value22	Value23 Value24	.
'             .
' .		..	  .	..		..		..		..		..
'
' Due to the internal storage scheme of ZBASIC arrays the 2D
' arrays are organized as array(1.. NumPars, 1 ... NumRecords)
' to allow easy access, i.e; Value14 = array(4,1)
'
' The method descriptions explain only relevant differences with the 
' methods of the PLX_DAQ_Table class
'
Class PLX_DAQ_Matrix Extends PLX_DAQ_Table

	' -----------------------------------------------------
	' Left column labels added (last parameter in method header)
	' 
	Sub SetTableProperties(ByVal TableNameString as String,  _
					  ByVal NumberOfPars as UnsignedInteger, ByVal NumberOfRecords as UnsignedInteger, _
					  ByVal TableRowPosition as UnsignedInteger, ByVal TableColumnPosition as UnsignedInteger, _
					  ByVal TableParameterLabels as String = "", ByVal TableItemLabels as String = "")
					  
		TableStart_Row = TableRowPosition
		TableStart_Column = TableColumnPosition
		TableName = TableNameString
		ParLabels = TableParameterLabels
		ItemLabels = TableItemLabels
		NumPars = NumberOfPars
		NumRecords = NumberOfRecords
	
	End Sub


	Sub WriteTable(ByVal TablePointer as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars*MaxNumRecords) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		
		Call WriteCell(TableStart_Row, TableStart_Column, TableName)
		Call GenerateParameterLabels
		Call GenerateItemLabels
		
		For CurrentRecordIndex = 1 To NumRecords
			For ParNumber = 1 To NumPars
				Call WriteCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber, CStr(DataRecordArray((CurrentRecordIndex-1)*NumPars + ParNumber)))
			Next ParNumber
		Next CurrentRecordIndex
		
		NumRecordsWritten = NumRecords
	
	End Sub
	
	' -----------------------------------------------------
	' Write a Single 1D array (dimensions: (2..NumPars) to the 
	' record with indicated row number in the MS-Excel table.
	' Note: A left-side label for each record is obligatory for matrices (2nd parameter)
	'
	Sub WriteTableRecord(ByVal RecordPointer as UnsignedInteger, _ 
						ByVal MatrixRowLabel as String, _
						ByVal CurrentXvalue as Single = 0.0)
	
		Dim DataRecordArray(2 To MaxPars) as Single Based RecordPointer
		Dim ParNumber as unsignedInteger
		
		RecordedTable = True
		
		If NumRecordsWritten = 0 Then
			Call WriteCell(TableStart_Row, TableStart_Column, TableName)
			Call GenerateParameterLabels
		End If

		Call WriteCell(TableStart_Row + 2 + NumRecordsWritten, TableStart_Column, MatrixRowLabel)
		Call WriteCell(TableStart_Row + 2 + NumRecordsWritten, TableStart_Column + 1, CStr(CurrentXvalue))
		For ParNumber = 2 To NumPars
			Call WriteCell(TableStart_Row + 2 + NumRecordsWritten, TableStart_Column + ParNumber, CStr(DataRecordArray(ParNumber)))
		Next ParNumber
	
		NumRecordsWritten = NumRecordsWritten + 1
	
	End Sub
	
	
	Sub ReadTable(ByVal TablePointer as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars*MaxNumRecords) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		Dim Flag as Boolean
		
		For CurrentRecordIndex = 1 To NumRecordsWritten
			For ParNumber = 1 To NumPars
				Call ValueS(ReadCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber), DataRecordArray((CurrentRecordIndex-1)*NumPars + ParNumber), Flag)
			Next ParNumber
		Next CurrentRecordIndex
	
	End Sub	


	Sub ReadTableRecord(ByVal TablePointer as UnsignedInteger, ByVal RowIndex as UnsignedInteger)
	
		Dim DataRecordArray(1 To MaxPars) as Single Based TablePointer
		Dim ParNumber as unsignedInteger
		Dim Flag as Boolean
		
		For ParNumber = 1 To NumPars
			Call ValueS(ReadCell(TableStart_Row + 2 + RowIndex - 1, TableStart_Column + ParNumber), DataRecordArray(ParNumber), Flag)
		Next ParNumber
	
	End Sub
	

	Sub Duplicate(ByVal TableRow as UnsignedInteger, ByVal TableColumn as UnsignedInteger, ByVal AltName as String = "")
	
		Dim i as UnsignedInteger
		Dim j as UnsignedInteger
		
		If AltName = "" Then
			Call WriteCell(TableRow, TableColumn, ReadCell(TableStart_Row, TableStart_Column))
		Else
			Call WriteCell(TableRow, TableColumn, AltName)
		End If
	
		For j = 0 To NumRecordsWritten
			For i = 1 To Numpars + 1
				Call WriteCell(TableRow + j + 1, TableColumn + i - 1, ReadCell(TableStart_Row + j + 1, TableStart_Column + i - 1))
			Next i
		Next j
	
	End sub
	
	
	Sub ClearTableData(ByVal EmptyString as String = "-")
	
		Dim ParNumber as unsignedInteger
		Dim CurrentRecordIndex as UnsignedInteger
		
		For CurrentRecordIndex = 1 To NumRecords
			For ParNumber = 1 To NumPars
				Call WriteCell(TableStart_Row + 2 + CurrentRecordIndex - 1, TableStart_Column + ParNumber, EmptyString)
			Next ParNumber
		Next CurrentRecordIndex
		
	End Sub
	
	
Protected:


	Sub GenerateParameterLabels()
		
		Dim ColonIndex as Byte
		Dim TextList as String = ParLabels
		Dim PickLabel as String
		Dim ColumnPointer as UnsignedInteger = TableStart_Column
		Dim i as UnsignedInteger
	
		If ParLabels = "" then
			Call WriteCell(TableStart_Row + 1, TableStart_Column, "Item:")
			For i = 1 To NumPars 
				Call WriteCell(TableStart_Row + 1, TableStart_Column + i, "Par " & CStr(i) & ":")
			Next i
		Else
			Do
				ColonIndex = Strfind(TextList,",")
				If ColonIndex <> 0 Then
					PickLabel = Left(TextList, ColonIndex - 1)
					Call WriteCell(TableStart_Row + 1, ColumnPointer, PickLabel)
					ColumnPointer = ColumnPointer + 1
					TextList = Mid(TextList, ColonIndex + 1)
				Else
					Call WriteCell(TableStart_Row + 1, ColumnPointer, TextList)
				End If
			Loop Until ColonIndex = 0
		End If
		
	End Sub

	' -----------------------------------------------------
	' Generate item labels on the left side of the table:
	' - if ItemLabels is empty then "Item i" type labels 
	'   are generated automatically in each row i,
	' - if ItemLabels is a comma-separated string then
	'   the labels between commas are put in each row
	' Labels are defined using SetTableProperties
	'
	Sub GenerateItemLabels()

		Dim ColonIndex as Byte
		Dim TextList as String = ItemLabels
		Dim PickLabel as String
		Dim RowPointer as UnsignedInteger = TableStart_Row + 2
		Dim i as UnsignedInteger
	
		If ItemLabels = "" then
			For i = 1 To NumRecords
				Call WriteCell(RowPointer - 1 + i, TableStart_Column, "#" & CStr(i))
			Next i
		Else
			Do
				ColonIndex = Strfind(TextList,",")
				If ColonIndex <> 0 Then
					PickLabel = Left(TextList, ColonIndex - 1)
					Call WriteCell(RowPointer, TableStart_Column, PickLabel)
					RowPointer = RowPointer + 1
					TextList = Mid(TextList, ColonIndex + 1)
				Else
					Call WriteCell(RowPointer, TableStart_Column, TextList)
				End If
			Loop Until ColonIndex = 0
		End If

	End Sub

	' -----------------------------------------------------
	' Additional object state variable describing user-supplied
	' items description labels as defined via
	' SetTableProperties method
	'
	Dim ItemLabels as String 
	
	
End Class





'**********************************************************
' Program Body
' Example program 1
' Generates numerical data (x, Sin(x) and Cos(x)) and 
' exports this to MS-Excel as an annotated matrix at the user's request
' Some simple operations can be managed via the PLX-DAQ controller window


'Create a controller and an MS-Excel matrix object
Dim MyController as PLX_DAQ_Controller
Dim MyMatrix as PLX_DAQ_Matrix

'Variable for storing controller checkbox information
Dim CheckBoxByte as Byte

Dim i as UnsignedInteger

Sub Main()
	
	' -----------------------------------------------------------
	' Define a set of datapoints
	' These could be the result of for example a datalogging application
	' XYData(1,*) contains the independent variables
	' XYData(2,*) contains the SIN function application
	' XYData(3,*) contains the COS function application
	'
	Const NumberOfParameters as UnsignedInteger = 3
	Const NumberOfDataRecords as UnsignedInteger = 20
	Dim XYData (1 To NumberOfParameters, 1 To NumberOfDataRecords) as Single
	For i = 1 to NumberOfDataRecords
		XYData(1, i) = 0.5 * CSng(i) 
		XYData(2, i) = Sin(XYData(1, i))
		XYData(3, i) = Cos(XYData(1, i))
	Next i
	
	'Set default controller view
	Call MyController.SetControllerProperties("Create Matrix", "Create records", _
											  "Clear Matrix", "Duplicate Matrix", _
											  " - Ready for Input - ")
	
	'Clear the MS-Excel worksheet
	Call MyMatrix.ClearSheet
	
	'Write welcome message at first row
	Call MyMatrix.WriteCell(1, 1, "Copy Matrix via ZBASIC array test")	

	'Define the properties of the MS-Excel matrix
	'(Note the selection of theSetTableproperties method from the PLX_DAQ_Matrix class 
	' by a dummy string as the seventh parameter)
	Call MyMatrix.SetTableProperties("My Matrix", NumberOfParameters, NumberOfDataRecords, _
									 5, 1, "item nr:,x,sin(x),cos(x)", "") 												   
	
	Do
	
		' Show controller properties
		Call MyController.UpdateController
		
		' Just wait until a checkbox is ticked by the user					   
		Do 
		Loop While Not MyController.ReadCheckBoxes(CheckBoxByte)
		
		' Take action depending on checkbox setting
		If CheckBoxByte = &B00000001 Then
			' Write the data array directly to MS-Excel
			Call MyController.WriteControllerMessage("Writing matrix ")
			Call MyMatrix.WriteTable(XYData.DataAddress)
		ElseIf CheckBoxByte = &B00000010 Then
			' Write the data array but now using separate records
			' This is generally the more memory-efficient way since in that case
			' the independent variables now stored in XYData(1,*) actually 
			' need not to be stored and thus RAM memory is conserved.
			Call MyController.WriteControllerMessage("Writing matrix as separate records")
			' Before writing a new table using records a Rewind is always necessary
			Call MyMatrix.Rewind
			For i = 1 To NumberOfDataRecords
				' Write matrix per record ( = row)
				' The object keeps track of the number of records written
				Call MyMatrix.WriteTableRecord(XYData(2,i).DataAddress, CStr(i), 0.5 * CSng(i))
			Next i
		ElseIf CheckBoxByte = &B00000100 Then
			' Clear the numerical data in the matrix
			Call MyController.WriteControllerMessage("Clearing matrix contents")
			Call MyMatrix.ClearTableData
		ElseIf CheckBoxByte = &B00001000 Then
			' Duplicate the matrix contents at position row=30, column=1
			' and give the matrix a different name.
			Call MyController.WriteControllerMessage("Duplicating matrix: this is a *slow* operation!")
			Call MyMatrix.Duplicate(30,1,"Copied Matrix")
		End If 
		
		' Clear checkboxes: ready for next action
		Call MyController.ResetCheckBoxes
		
	Loop

End Sub
